#include "tabwidget.h"
#include <config_dialog.h>

TabWidget::TabWidget()
{
	tabbar = new TabBar(this);
	setTabBar(tabbar);

	connect(tabbar, SIGNAL(contextMenu(int, const QPoint&)),
			SLOT(onContextMenu(int, const QPoint&)));
	connect(tabbar, SIGNAL(deleteTab(int)),
			SLOT(onDeleteTab(int)));
	connect(tabbar, SIGNAL(wheelEventSignal(QWheelEvent*)),
			SLOT(wheelEvent(QWheelEvent*)));
	connect(tabbar, SIGNAL(moveTab(int, int)),
			SLOT(moveTab(int, int)));
}

void TabWidget::closeEvent(QCloseEvent* e)
{
	kdebugf();
	QWidget* current = currentPage();
	delete current;

	if (count() > 0)
		e->ignore();
	else
		e->accept();
}

void TabWidget::chatKeyPressed(QKeyEvent* e, CustomInput* k, bool &handled)
{
	handled = true;
	if (HotKey::shortCut(e, "ShortCuts", "MoveTabLeft"))
		moveTabLeft();
	else if(HotKey::shortCut(e, "ShortCuts", "MoveTabRight"))
		moveTabRight();
	else if(HotKey::shortCut(e, "ShortCuts", "SwitchTabLeft"))
		switchTabLeft();
	else if(HotKey::shortCut(e, "ShortCuts", "SwitchTabRight"))
		switchTabRight();
	else
		handled = false;
}

void TabWidget::onContextMenu(int id, const QPoint& pos)
{
	kdebugf();
	emit contextMenu(page(id), pos);
}

void TabWidget::moveTab(int from, int to)
{
	kdebugf();
	qDebug("from==%i, to==%i\n", from, to);
	QString tablabel=label(from);
	QWidget *w=page(from);
	QIconSet tabiconset = tabIconSet(w);
	QString tabtooltip = tabToolTip(w);
	bool current=(w==currentPage());
	blockSignals(true);
	removePage(w);

	insertTab(w, tabiconset, tablabel, to);

	if(current)
		setCurrentPage(to);

	blockSignals(false);
}

void TabWidget::onDeleteTab(int id)
{
	emit deleteTab(page(id));
}

void TabWidget::wheelEvent(QWheelEvent* e)
{
	if (e->delta() > 0)
		switchTabLeft();
	else
		switchTabRight();
}

void TabWidget::switchTabLeft()
{
	if (currentPageIndex() == 0)
		setCurrentPage(count() - 1);
	else
		setCurrentPage(currentPageIndex()-1);
}

void TabWidget::switchTabRight()
{
	if (currentPageIndex()==(count()-1))
		setCurrentPage(0);
	else
		setCurrentPage(currentPageIndex()+1);
}

void TabWidget::moveTabLeft()
{
	if (currentPageIndex() == 0)
		moveTab(0, count() - 1);
	else
		moveTab(currentPageIndex(), currentPageIndex() - 1);
}

void TabWidget::moveTabRight()
{
	if (currentPageIndex() == (count() - 1))
		moveTab(count() - 1, 0);
	else
		moveTab(currentPageIndex(), currentPageIndex() + 1);
}

TabBar::TabBar(QWidget *parent, char *name)
	: QTabBar(parent, name)
{
}

void TabBar::mousePressEvent(QMouseEvent* e)
{
	QTab* selectedtab = selectTab(e->pos());

	if (selectedtab)
	{
		if(e->button() == LeftButton)
		{
			// zmieniamy kursor na "apk"
			setCursor(QCursor(Qt::SizeAllCursor));
			// zapamitujemy identyfikator karty rdowej
			movingTabId = selectedtab->identifier();
		}
		else if (e->button() == RightButton)
			emit contextMenu(indexOf(selectedtab->identifier()), mapToGlobal(e->pos()));
		else if (e->button() == MidButton)
			emit deleteTab(indexOf(selectedtab->identifier()));
	}

	QTabBar::mousePressEvent(e);
}

void TabBar::mouseReleaseEvent(QMouseEvent* e)
{
	QTab* selectedtab = selectTab(e->pos());

	if (selectedtab)
	{
		// pozycja karty docelowej
		int destinationTabId = selectedtab->identifier();
		// przywracamy kursor
		setCursor(QCursor(Qt::ArrowCursor));

		if(e->button() == LeftButton && indexOf(movingTabId) != -1 && destinationTabId != movingTabId)
			// zamieniamy miejcami kart rdow z docelow
			emit moveTab(indexOf(movingTabId), indexOf(destinationTabId));
	}

	QTabBar::mouseReleaseEvent(e);
}

void TabBar::wheelEvent(QWheelEvent *e)
{
	emit wheelEventSignal(e);
}
