#ifndef TABS_TABWIDGET_H
#define TABS_TABWIDGET_H

#include <debug.h>

#include <qtabwidget.h>
#include <qtabbar.h>
#include <qcursor.h>

#include "custom_input.h"

class TabBar: public QTabBar
{
	Q_OBJECT

	protected:
		virtual void mousePressEvent(QMouseEvent* e);
		virtual void mouseReleaseEvent(QMouseEvent* e);
		virtual void wheelEvent(QWheelEvent* e);

	signals:
		void contextMenu(int id, const QPoint& pos);
		void deleteTab(int);
		void wheelEventSignal(QWheelEvent* e);
		void moveTab(int from, int to);

	private:
		int movingTabId;

	public:
		TabBar(QWidget *parent = 0, char *name = 0);
};

class TabWidget: public QTabWidget
{
	Q_OBJECT

	public:
		TabWidget();

		/**
		* Zmienia aktualn kart na t po lewej stronie
		*/
		void switchTabLeft();

		/**
		* Zmienia aktualn kart na t po prawej stronie
		*/
		void switchTabRight();

		void moveTabLeft();
		void moveTabRight();
	private:
		TabBar* tabbar;
	private slots:
		void onContextMenu(int id, const QPoint& pos);

		/**
		* Slot zostaje wywoany w celu zmiany pozycji karty.
		* @param from pozycja karty rdowej.
		* @param to pozycja karty docelowej.
		*/
		void moveTab(int from, int to);
		void onDeleteTab(int);
	protected:
		virtual void closeEvent(QCloseEvent* e);
	public slots:
		virtual void chatKeyPressed(QKeyEvent* e, CustomInput* k, bool &handled);

		/**
		* Slot zostaje wywoany w momencie ruchu kkiem myszy.
		* @param e wskanik obiektu opisujcego to zdarzenie.
		*/
		virtual void wheelEvent(QWheelEvent* e);
	signals:
		void contextMenu(QWidget* w, const QPoint& pos);
		void deleteTab(QWidget* w);
};

#endif
